#include <Adafruit_BusIO_Register.h>
#include <Adafruit_I2CDevice.h>
#include <Adafruit_I2CRegister.h>
#include <Adafruit_SPIDevice.h>

#define I2C_ADDRESS 0x60
Adafruit_I2CDevice mcp4716 = Adafruit_I2CDevice(I2C_ADDRESS);
unsigned char buffer[8];
unsigned char i;
char hexchars[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
unsigned int dacval;
char command_str[12];
unsigned char command_count=0;
char command_char;
bool command_complete=false;
#define enable 8        //output to control ENA, DIS

//print the value as a hex string
void hexit(unsigned char hval)
{
  unsigned char i;
  Serial.print(hexchars[hval/16]);
  hval = hval % 16;
  Serial.print(hexchars[hval]); 
}

// **************************
//this function breaks the  0-1023 decimal  DAC value into two bytes
//formatted the way the MCP4716  requires them.
void set_dacval(unsigned int dv)
{
    buffer[1] = (dv >> 2) & 0xff;  //first I2C dac data byte is upper byte of 10-bit value
    buffer[2] =  (dv << 6)  & 0xff;      //2nd byte contains the two LSbits .. shifted into MSbit position
}


// ***************************
//this function is only required ONCE -- and has been previously run at the factory so you do not need to
//run it. Its purpose is to configure the DAC's reference voltage, turn off its Power-Down function, and
//set the DAC's default power-up value to zero.   THe power-up value can be changed by setting dacval to
//something other than 0, and executing this function once.
void initial_DAC_setup()
{
    Serial.println("Settup up DAC");
    buffer[0] = 0x78;             //write ALL Memory to set DAC GAIN=1, VREF=11, PD=00
    dacval = 0;                   //store zero in EEPROM so DAC powers up at zero output
    set_dacval(dacval);           //shift bits into position
    mcp4716.write(buffer, 3);       //write 3 bytes
    delay(75);                      //DAC takes 50ms to write setup values to EEPROM so wait
   
}

void show_instructions()
{
    Serial.println("To operate, connect Arduino SCL, SDA, GND to same VCPWR signals.  And connect");
    Serial.println("Arduino pin 8  to VCPWR /ENA  input");  Serial.println();
    Serial.println("This simple program allows you to enter 10-bit DAC values .. as decimal 0-1023");
    Serial.println("and send them to the VCPWR I2C DAC.  The value 1023 corresponds to the output");
    Serial.println("set previously as the Full Scale value.  Finish each command with ENTER key:");
    Serial.println("COMMANDS:");
    Serial.println("DACxxxx     Set DAC to decimal value");
    Serial.println("EEPxxxx     Set DAC to decimal value, and power-up value to same");
    Serial.println("ENA         Enable VCPWR output (connected to Arduino pin 8)");
    Serial.println("DIS         DISable VCPWR output (Arduino pin 8)");
    Serial.println("?           Show The Instructions"); 
}

// ****************************
void setup() {                            //config serial monitor to operate at 38.4k baud
   Serial.begin(38400);
   Serial.println("VCPWR I2C DAC test");

  pinMode(enable, OUTPUT);
  digitalWrite(enable, LOW);          //ehis enables the output
  if (!mcp4716.begin()) {
    Serial.print("Did not find device at 0x");
    Serial.println(mcp4716.address(), HEX);
  }
  else 
  {
    mcp4716.setSpeed(100000);         //speed to 100k
    Serial.println("Device found");
    show_instructions();
    //initial_DAC_setup();          //this has been run at the factory so is not needed again
  }
}


// ****************************
//read DAC command -- diagnostic only.  Dumps the DAC's volatile and non-volatile settings
void RDAC()
{
    mcp4716.read(buffer,6);       //read the 6 bytes of status info
    for (i=0;i<6;i++)             //spit it out as hex values for visual analysis
    {
      hexit(buffer[i]);  
      Serial.print("  ");  
    }
    Serial.println(); 
}


void loop() {

byte cmdbyte;
bool valid_command;

  //get a command from the Serial monitor window
  if (Serial.available())                             //collect command chars til user presses ENTER
  {
      if (command_count < sizeof(command_str))        //if there is room to store the command continue
      {
          command_char = Serial.read();               //read the char that user entered
          if (command_char == 0x0d)                   //is it a carriage ret  (ENTER)?
          {
              command_char = 0;                       //make it a null so it terminates the inpur string
              command_complete = true;                //if so, command is complete, so execute
          }
          command_str[command_count++] = toupper(command_char);   //if not end of command, keep collecting
          if (command_char == 0) Serial.println();
          else Serial.print(command_char);
      }
      else                                            //user has entered too many characters. flush this command
      {
        command_count = 0;
        command_complete = false;
      }
  }
  if (command_complete)                                 //do we have a command to execute?
  {
      cmdbyte = 0;
      valid_command = false;
      if (command_str[0] == 'D' && command_str[1] == 'A'  && command_str[2] == 'C')  
      {
        cmdbyte = 0x58;  //writes to volatile DAC register
        valid_command = true;
      }
      if (command_str[0] == 'E' && command_str[1] == 'E'  && command_str[2] == 'P')
      {
        cmdbyte = 0x78;  //writes to EEPROM + volatile DAC register 
        valid_command = true;
      }
      if (command_str[0] == 'E' && command_str[1] == 'N'  && command_str[2] == 'A') 
      {
        digitalWrite(enable, LOW);  //enable the output
        valid_command = true;
      }
      if (command_str[0] == 'D' && command_str[1] == 'I'  && command_str[2] == 'S')
      {
        digitalWrite(enable, HIGH);  //disable the output
        valid_command = true;
      }
      if (command_str[0] == '?')
      {
        show_instructions();
        valid_command = true;
      }

      if (valid_command)
      {
          if (cmdbyte != 0)                                 //if user has entered a valid command     
          {
               dacval = atoi(&command_str[3]);             //extract the integer number user entered
              if (dacval <= 1023)                         //1023 is full scale
              {
                set_dacval(dacval);                    //setup the i2c buffer to command DAC
                buffer[0] = cmdbyte;                   //command to update the DAC
                mcp4716.write(buffer, 3);              //send the command    
              }
              else  Serial.println("Error .. bad value, must be 0 to 1023");
          }     
      }
      else
      {
          Serial.println("Error .. command must start with DAC or  EEP");
      }
      command_complete= false;
      command_count = 0;
  }
  
}
